(document:surround "/std/base")
(document:insert "/std/functions")
(document:envelop with-translation _ "alterator-printers")

(define (get-printers)
  (woo-catch/message
   (thunk
     (woo-list-names "/cupsd/printers"))))

(define (get-server)
  (woo-catch/message
   (thunk
     (woo-read-first "/cupsd/server"))))

(define (getprinter name)
  (woo-catch/message
   (thunk
      (woo-read-first "/cupsd" 'prnname name))))

;current printer data
(define printer-type (make-cell #f))
(define printer-data (make-cell #f))

;edit/new data
(define printer-status (make-cell #f))
(define printer-uri (make-cell #f))
(define printer-driver (make-cell #f))

;edit/new mode
(define mode (make-cell 'edit))

(define (redraw-status-button)
  (state-button text (if (cell-ref printer-status)
                         (_ "stop printer")
                         (_ "start printer")))
  (state-button pixmap (if (cell-ref printer-status)
                           "printers/pause.png"
                           "printers/play.png")))

(define (update-status-button prn)
  (cell-set! printer-status
             (string=? (woo-get-option prn 'status) "started"))
  (redraw-status-button))

(define (extract-model prn)
  (let* ((driver (woo-get-option prn 'driver))
         (name (cond-cadr (string-cut driver #\;))))
    (or name (_ "Unknown model"))))

(define (update-connection-button mdl)
  (connection-button text
              (string-append
               (cond
                ((string=? (cell-ref printer-type) "smb") (_ "Samba"))
                ((string=? (cell-ref printer-type) "lcl") (_ "Local"))
		(else (_"Unknown connection")))
               "/"
	       (or mdl (_ "Unknown model")))))
  
(define (update-connection uri)
    (cond
     ((string-prefix? "smb://" uri) (cell-set! printer-type "smb"))
     ((string-prefix? "parallel:" uri) (cell-set! printer-type "lcl"))
     ((string-prefix? "usb:" uri) (cell-set! printer-type "lcl"))
     (else (cell-set! printer-type "sel"))))

(define (fill-info name)
  (let ((printer (getprinter name)))
        (cell-set! printer-data printer)
        (cell-set! printer-uri (woo-get-option printer 'uri))
        (cell-set! printer-driver (string-cut (woo-get-option printer 'driver) #\;))
  	(printername text name)
        (printerdesc text (woo-get-option printer 'description))
        (update-connection (woo-get-option printer 'uri))
        (update-connection-button (extract-model printer))
  	(update-status-button printer)))

(define (mark-default name default)
  (cons name
        (if (string=? name default) "theme:radio-on" "theme:radio-off")))

(define (do-default)
  (define (extract-text x) (car (vector-ref x 0)))
  (woo-catch/message
   (thunk
     (let ((old-idx (string-list-index (woo-get-option (get-server)
                                                       'default)
                                       (map extract-text (printerlist rows))))
           (new-idx (printerlist current)))
       (woo-write "/cupsd/server" 'default (printerlist text))
       (printerlist row-item old-idx (cons #f "theme:radio-off")
                    row-item new-idx (cons #f "theme:radio-on"))))))

(define (do-test-page)
  (woo-catch/message
   (thunk
     (woo-write "/cupsd/test_page" 'printer (printerlist text)))))

(define (do-edit)
  (switch-state))

(define (do-new)
  (switch-state)
  ;new specific actions
  (cell-set! mode 'new)
  
  (printername activity #t)
  (printerdesc activity #t)
  (printername text "")
  (printerdesc text "")
  (cell-set! printer-data '())
  (cell-set! printer-driver #f)
  (cell-set! printer-uri #f)
  (cell-set! printer-type "sel")
  (connection-button text (_ "Create connection"))
  
  (cell-set! printer-status #t)
  (redraw-status-button)
  
  (printerlist append-row (cons (_ "NEW PRINTER") "theme:radio-off"))
  (printerlist current (- (printerlist count) 1)))

;TODO: add support for change type here
(define (change-connection)
  (error-message visibility #f)
  (let ((result (document:popup (string-append "/printers/" (cell-ref printer-type))
                                'printer-uri (or (cell-ref printer-uri)
                                                 (woo-get-option (cell-ref printer-data) 'uri))
                                'printer-driver (or (cell-ref printer-driver)
                                                    (string-cut (woo-get-option (cell-ref printer-data) 'driver) #\;)))))
    (and (pair? result)
         (begin (cell-set! printer-uri (car result))
                (cell-set! printer-driver (cdr result))
		(update-connection (cell-ref printer-uri))
		(update-connection-button (cond-cadr (cell-ref printer-driver)))))))

(define (check-buttons)
  ((widgets edit-button
           delete-button
           testpage-button
           default-button)
	   activity (> (printerlist count) 0)))

(define (switch-state)
  (error-message visibility #f)
  (printerlist activity (not (printerlist activity)));switch
  (let ((state (printerlist activity)))
    (group1 visibility state)
    (group2 visibility (not state))
    (printerdesc activity (not state))
    
    (for-each
     (lambda(w) (w activity (not state)))
     (list connection-button
           state-button))

    (for-each
     (lambda(w) (w activity state))
     (list testpage-button
           default-button)))
    (check-buttons))

(define (write-error msg)
  (error-message visibility #t)
  (error-message text (string-append "<font color=\"red\"><b>" msg "</b></font>"))
  #f)

(define (cleanup-model-name vendor model)
  (and vendor
       model
       (let* ((cutted (substring model (string-length vendor)
                                 (string-length model)))
              (start (string-skip cutted #\space)))
                   (substring cutted start (string-length cutted)))))

(define (get-driver)
  (let* ((vendor (car (cell-ref printer-driver)))
         (model (cleanup-model-name vendor (cadr (cell-ref printer-driver)))))
    (woo-get-option
     (woo-read-first (string-append "/printer_drivers/" vendor "/" model))
     'id)))

(define (modify-printer)
  (cond
   ((empty-string? (printername text))
    (write-error (_ "You should define printer name")))
   ((not (string-match "^([0-9a-zA-Z_-])+$" (printername text)))
    (write-error (_ "Printer name doesn't match required pattern '^([0-9a-zA-Z_-])+$'")))
   ((and (eq? (cell-ref mode) 'new) (not (cell-ref printer-driver)))
    (write-error (_ "You should define printer model")))
   ((and (eq? (cell-ref mode) 'new) (empty-string? (cell-ref printer-uri)))
    (write-error (_ "You shoud define printer connection")))
   (else
    (splash-message (_ "Generating printer driver..."))
    (begin-1
     (woo-catch/message
      (thunk
        (document:release)
        (woo-write "/cupsd"
            'prnname (printername text)
            'uri (cell-ref printer-uri)
            'ppd (or (caddr (cell-ref printer-driver)) (get-driver))
            'description (printerdesc text)
            'status (if (cell-ref printer-status) "started" "stopped"))))
     (splash-message)))))

(define (do-delete)
     (and (eq? 'yes (document:popup-warning
                   (string-append (_ "Are you really want to remove printer ")
                                  (printerlist text)
                                  " ?")
                   'no
                   'yes))
          (woo-catch/message
           (thunk
             (woo-delete "/cupsd" 'prnname (printerlist text))
             (printerlist remove (printerlist current))
	     (check-buttons)
	     (and (> (printerlist count) 0)
	          (begin (printerlist current 0))
	                 (fill-info (printerlist text)))))))

(define (generate-dump)
  (splash-message (_ "Generating drivers cache..."))
  (begin-1
   (woo-catch/message
    (thunk
     (document:release)
     (woo-write "/cupsd/dodump")))
   (splash-message)))

(hbox
       (vbox
             (document:id printerlist (listbox
	                                       header (vector (_ "Printers"))
                                               (when selected
                                                 ((widgets testpage-button
                                                           default-button
                                                           edit-button
                                                           delete-button) activity #t)
                                                 (fill-info (printerlist text)))))
             (hbox
              (document:id testpage-button (button (_ "Test page")
                                                   activity #f
                                                   (when clicked (do-test-page))))
              (document:id default-button (button (_ "Made default")
                                                  activity #f
                                                  (when clicked (do-default))))
              (button (_ "Generate drivers cache")
                                                  (when clicked (generate-dump)))))
       (vbox
             (groupbox (_ "Printer options")
                             (hbox
                                   (label (_ "Name:"))
                                   (document:id printername (edit ""
                                                                  activity #f
                                                                  (when changed
                                                                   (printerlist row-item
                                                                                (printerlist current)
                                                                                (printername text))))))
                             (hbox
                                   (label (_ "Description:"))
                                   (document:id printerdesc (edit ""
                                                                  activity #f)))
                             (document:id connection-button
                                          (button (_ "Connection/Model")
                                                  activity #f
                                                  (when clicked (change-connection))))
                             (document:id state-button
                                          (button (_ "stop printer")
                                                               pixmap "printers/pause.png"
                                                               activity #f
                                                               (when clicked
                                                                (cell-set!  printer-status
                                                                            (not (cell-ref printer-status)))
                                                                (redraw-status-button))))
                             (document:id error-message (label "" visibility #f)))

             (document:id group1 (hbox
                                  (document:id edit-button (button (_ "Edit")
                                                                   activity #f
                                                                   (when clicked (do-edit))))
                                  (document:id new-button (button (_ "New")
                                                                  (when clicked (do-new))))
                                  (document:id delete-button (button (_ "Delete")
                                                                     activity #f
                                                                     (when clicked (do-delete))))))          
             (document:id group2 (hbox
                                       visibility #f
                                       (document:id accept-button (button (_ "Accept")
                                                                          (when clicked
                                                                           (and (modify-printer)
                                                                                (begin
                                                                                  (and (eq? (cell-ref mode) 'new)
                                                                                       (begin
                                                                                         (cell-set! mode 'edit)
                                                                                         (printername activity #f)))
                                                                                  (fill-info (printerlist text))
                                                                                  (switch-state))))))
                                       (document:id discard-button (button (_ "Discard")
                                                                           (when clicked
                                                                            (case (cell-ref mode)
                                                                              ((new)
                                                                               (cell-set! mode 'edit)
                                                                               (printername activity #f)
                                                                               (printerlist remove (printerlist current))
                                                                               (and (> (printerlist count) 0) (printerlist current 0))))
                                                                            (and (> (printerlist count) 0)(fill-info (printerlist text)))
                                                                            (switch-state))))))))
	(button (_ "Quit") (when clicked (document:end)))


(document:root
  (when loaded
   (let ((data (get-server)))
     (printerlist rows (map (lambda (name) (mark-default name (woo-get-option data 'default)))
                            (or (get-printers) '())))
     (if (woo-get-option data 'nodump #f)
         (generate-dump))
     (if (woo-get-option data 'nocups #f)
         (document:popup-critical (_ "Service cupsd is stopped") 'ok)))))
